==> scummvm_0.6.0.orig.tar.gz: scummvm-0.6.0/sky/music/musicbase.cpp <==
/* ScummVM - Scumm Interpreter
 * Copyright (C) 2003-2004 The ScummVM project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /cvsroot/scummvm/scummvm/sky/music/musicbase.cpp,v 1.19 2004/01/06 12:45:33 fingolfin Exp $
 *
 */

#include "sky/music/musicbase.h"
#include "sky/disk.h"
#include "common/util.h"

namespace Sky {

MusicBase::MusicBase(Disk *pDisk, OSystem *system) {

	_musicData = NULL;
	_allowedCommands = 0;
	_skyDisk = pDisk;
	_currentMusic = 0;
	_musicVolume = 127;
	_system = system;
	_numberOfChannels = _currentMusic = 0;
	_mutex = _system->create_mutex();
}

MusicBase::~MusicBase(void) {

	if (_musicData)
		free(_musicData);
}

void MusicBase::loadSection(uint8 pSection) {

	_system->lock_mutex(_mutex);
	if (_currentMusic)
		stopMusic();
	if (_musicData)
		free(_musicData);
	_currentSection = pSection;
	_musicData = _skyDisk->loadFile(_driverFileBase + FILES_PER_SECTION * pSection, NULL);
	_allowedCommands = 0;
	_musicTempo0 = 0x78; // init constants taken from idb file, area ~0x1060
	_musicTempo1 = 0xC0;
	_onNextPoll.doReInit = false;
	_onNextPoll.doStopMusic = false;
	_onNextPoll.musicToProcess = 0;
	_tempo = _aktTime = 0x10001;
	_numberOfChannels = _currentMusic = 0;
	setupPointers();
	startDriver();
	_system->unlock_mutex(_mutex);
}

bool MusicBase::musicIsPlaying(void) {

	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++)
		if (_channels[cnt]->isActive())
			return true;
	return false;
}

void MusicBase::musicCommand(uint16 command) {

	if (_musicData == NULL) {
		debug(1,"Got music command but driver is not yet loaded");
		return ;
	}
	if ((command >> 8) > _allowedCommands) {
		debug(1,"got musicCommand %d while expecting <= %d", command >> 8, _allowedCommands);
		return ;
	}
	switch(command >> 8) {
	case 0: 
		debug(1,"Music: got call to startAdlibDriver(). Not necessary in this implementation.");
		break;
	case 1: 
		debug(1,"Music: got call to stopDriver(). Not necessary in this implementation.");
		break;
	case 2:
		debug(1,"Music: got call to SetTempo(). Tempo is fixed in this implementation.");
		break;
	case 3: 
		debug(1,"Music: ignored direct call to driverPoll().");
		break;
	case 4: 
		startMusic(command & 0xFF);
		break;
	case 6:
		reinitFM();
		break;
	case 7:
		stopMusic();
		break;
	case 13: 
		setFMVolume(command & 0xFF); 
		break;
	default: 
		debug(1,"musicCommand %d ignored.",command >> 8);
	}
}

void MusicBase::setFMVolume(uint16 param) {

	_musicVolume = param;
	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++)
		_channels[cnt]->updateVolume(_musicVolume);
}

void MusicBase::stopMusic(void) {

	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++) {
		_channels[cnt]->stopNote();
		delete _channels[cnt];
	}
	_numberOfChannels = 0;
}

void MusicBase::updateTempo(void) {

	uint16 tempoMul = _musicTempo0 * _musicTempo1;
	uint16 divisor = 0x4446390/ 23864;
	_tempo = (tempoMul / divisor) << 16;
	_tempo |= (((tempoMul%divisor) << 16) | (tempoMul / divisor)) / divisor;
}

void MusicBase::loadNewMusic(void) {

	uint16 musicPos;
	if (_onNextPoll.musicToProcess > _musicData[_musicDataLoc]) {
		error("Music %d requested but doesn't exist in file.", _onNextPoll.musicToProcess);
		return;
	}
	if (_currentMusic != 0)
		stopMusic();

	_currentMusic = _onNextPoll.musicToProcess;

	if (_currentMusic != 0) {
		musicPos = (_musicData[_musicDataLoc + 2] << 8) | _musicData[_musicDataLoc+1];
		musicPos += _musicDataLoc+((_currentMusic-1) << 1);
		musicPos = ((_musicData[musicPos+1] << 8) | _musicData[musicPos]) + _musicDataLoc;

		_musicTempo0 = _musicData[musicPos];
		_musicTempo1 = _musicData[musicPos+1];

		setupChannels(_musicData + musicPos + 2);

		updateTempo();
	}
}

void MusicBase::pollMusic(void) {

	_system->lock_mutex(_mutex);
	uint8 newTempo;
	if (_onNextPoll.doReInit) startDriver();
	if (_onNextPoll.doStopMusic) stopMusic();
	if (_onNextPoll.musicToProcess != _currentMusic)
		loadNewMusic();

	_aktTime += _tempo;

	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++) {
		newTempo = _channels[cnt]->process((uint16)(_aktTime >> 16));
		if (newTempo) {
			_musicTempo1 = newTempo;
			updateTempo();
		}
	}
	_system->unlock_mutex(_mutex);
	_aktTime &= 0xFFFF;
}

} // End of namespace Sky

==> scummvm_0.7.1.orig.tar.gz: scummvm-0.7.1/sky/music/musicbase.cpp <==
/* ScummVM - Scumm Interpreter
 * Copyright (C) 2003-2004 The ScummVM project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /cvsroot/scummvm/scummvm/sky/music/musicbase.cpp,v 1.21 2004/11/11 10:14:35 lavosspawn Exp $
 *
 */

#include "sky/music/musicbase.h"
#include "sky/disk.h"
#include "common/util.h"

namespace Sky {

MusicBase::MusicBase(Disk *pDisk, OSystem *system) {

	_musicData = NULL;
	_allowedCommands = 0;
	_skyDisk = pDisk;
	_currentMusic = 0;
	_musicVolume = 127;
	_system = system;
	_numberOfChannels = _currentMusic = 0;
	_mutex = _system->createMutex();
}

MusicBase::~MusicBase(void) {

	if (_musicData)
		free(_musicData);
}

void MusicBase::loadSection(uint8 pSection) {

	_system->lockMutex(_mutex);
	if (_currentMusic)
		stopMusic();
	if (_musicData)
		free(_musicData);
	_currentSection = pSection;
	_musicData = _skyDisk->loadFile(_driverFileBase + FILES_PER_SECTION * pSection);
	_allowedCommands = 0;
	_musicTempo0 = 0x78; // init constants taken from idb file, area ~0x1060
	_musicTempo1 = 0xC0;
	_onNextPoll.doReInit = false;
	_onNextPoll.doStopMusic = false;
	_onNextPoll.musicToProcess = 0;
	_tempo = _aktTime = 0x10001;
	_numberOfChannels = _currentMusic = 0;
	setupPointers();
	startDriver();
	_system->unlockMutex(_mutex);
}

bool MusicBase::musicIsPlaying(void) {

	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++)
		if (_channels[cnt]->isActive())
			return true;
	return false;
}

void MusicBase::musicCommand(uint16 command) {

	if (_musicData == NULL) {
		debug(1,"Got music command but driver is not yet loaded");
		return ;
	}
	if ((command >> 8) > _allowedCommands) {
		debug(1,"got musicCommand %d while expecting <= %d", command >> 8, _allowedCommands);
		return ;
	}
	switch(command >> 8) {
	case 0: 
		debug(1,"Music: got call to startAdlibDriver(). Not necessary in this implementation.");
		break;
	case 1: 
		debug(1,"Music: got call to stopDriver(). Not necessary in this implementation.");
		break;
	case 2:
		debug(1,"Music: got call to SetTempo(). Tempo is fixed in this implementation.");
		break;
	case 3: 
		debug(1,"Music: ignored direct call to driverPoll().");
		break;
	case 4: 
		startMusic(command & 0xFF);
		break;
	case 6:
		reinitFM();
		break;
	case 7:
		stopMusic();
		break;
	case 13: 
		setFMVolume(command & 0xFF); 
		break;
	default: 
		debug(1,"musicCommand %d ignored.",command >> 8);
	}
}

void MusicBase::setFMVolume(uint16 param) {

	_musicVolume = param;
	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++)
		_channels[cnt]->updateVolume(_musicVolume);
}

void MusicBase::stopMusic(void) {

	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++) {
		_channels[cnt]->stopNote();
		delete _channels[cnt];
	}
	_numberOfChannels = 0;
}

void MusicBase::updateTempo(void) {

	uint16 tempoMul = _musicTempo0 * _musicTempo1;
	uint16 divisor = 0x4446390/ 23864;
	_tempo = (tempoMul / divisor) << 16;
	_tempo |= (((tempoMul%divisor) << 16) | (tempoMul / divisor)) / divisor;
}

void MusicBase::loadNewMusic(void) {

	uint16 musicPos;
	if (_onNextPoll.musicToProcess > _musicData[_musicDataLoc]) {
		error("Music %d requested but doesn't exist in file.", _onNextPoll.musicToProcess);
		return;
	}
	if (_currentMusic != 0)
		stopMusic();

	_currentMusic = _onNextPoll.musicToProcess;

	if (_currentMusic != 0) {
		musicPos = (_musicData[_musicDataLoc + 2] << 8) | _musicData[_musicDataLoc+1];
		musicPos += _musicDataLoc+((_currentMusic-1) << 1);
		musicPos = ((_musicData[musicPos+1] << 8) | _musicData[musicPos]) + _musicDataLoc;

		_musicTempo0 = _musicData[musicPos];
		_musicTempo1 = _musicData[musicPos+1];

		setupChannels(_musicData + musicPos + 2);

		updateTempo();
	}
}

void MusicBase::pollMusic(void) {

	_system->lockMutex(_mutex);
	uint8 newTempo;
	if (_onNextPoll.doReInit) startDriver();
	if (_onNextPoll.doStopMusic) stopMusic();
	if (_onNextPoll.musicToProcess != _currentMusic)
		loadNewMusic();

	_aktTime += _tempo;

	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++) {
		newTempo = _channels[cnt]->process((uint16)(_aktTime >> 16));
		if (newTempo) {
			_musicTempo1 = newTempo;
			updateTempo();
		}
	}
	_system->unlockMutex(_mutex);
	_aktTime &= 0xFFFF;
}

} // End of namespace Sky
